@@ -45,7 +45,7 @@ static char *percentalerts[] = {
 
 int Monitor(mddev_dev_t devlist,
 	    char *mailaddr, char *alert_cmd,
-	    int period, int daemonise, int scan,
+	    int period, int daemonise, int scan, int oneshot,
 	    char *config)
 {
 	/*
@@ -97,6 +97,7 @@ int Monitor(mddev_dev_t devlist,
 		int err;
 		char *spare_group;
 		int active, working, failed, spare, raid;
+		int expected_spares;
 		int devstate[MD_SB_DISKS];
 		int devid[MD_SB_DISKS];
 		int percent;
@@ -108,17 +109,19 @@ int Monitor(mddev_dev_t devlist,
 	if (!mailaddr) {
 		mailaddr = conf_get_mailaddr(config);
 		if (mailaddr && ! scan)
-			printf("mdadm: Monitor using email address \"%s\" from config file\n",
+			fprintf(stderr, Name ": Monitor using email address \"%s\" from config file\n",
 			       mailaddr);
 	}
 	if (!alert_cmd) {
 		alert_cmd = conf_get_program(config);
 		if (alert_cmd && ! scan)
-			printf("mdadm: Monitor using program \"%s\" from config file\n",
+			fprintf(stderr, Name ": Monitor using program \"%s\" from config file\n",
 			       alert_cmd);
 	}
-	if (scan && !mailaddr && !alert_cmd)
+	if (scan && !mailaddr && !alert_cmd) {
+		fprintf(stderr, Name ": No mail address or alert command - not monitoring.\n");
 		return 1;
+	}
 
 	if (daemonise) {
 		int pid = fork();
@@ -149,6 +152,7 @@ int Monitor(mddev_dev_t devlist,
 			st->err = 0;
 			st->devnum = -1;
 			st->percent = -2;
+			st->expected_spares = mdlist->spare_disks;
 			if (mdlist->spare_group)
 				st->spare_group = strdup(mdlist->spare_group);
 			else
@@ -167,6 +171,7 @@ int Monitor(mddev_dev_t devlist,
 			st->err = 0;
 			st->devnum = -1;
 			st->percent = -2;
+			st->expected_spares = -1;
 			st->spare_group = NULL;
 			statelist = st;
 		}
@@ -174,6 +179,7 @@ int Monitor(mddev_dev_t devlist,
 
 
 	while (! finished) {
+		int new_found = 0;
 		struct state *st;
 
 		if (mdstat)
@@ -239,6 +245,16 @@ int Monitor(mddev_dev_t devlist,
 				st->err = 0;
 				continue;
 			}
+			if (st->utime == 0 && /* new array */
+			    mse &&	/* is in /proc/mdstat */
+			    mse->pattern && strchr(mse->pattern, '_') /* degraded */
+				)
+				alert("DegradedArray", dev, NULL, mailaddr, alert_cmd);
+
+			if (st->utime == 0 && /* new array */
+			    st->expected_spares > 0 && 
+			    array.spare_disks < st->expected_spares) 
+				alert("SparesMissing", dev, NULL, mailaddr, alert_cmd);
 			if (mse &&
 			    st->percent == -1 && 
 			    mse->percent >= 0)
@@ -321,6 +337,7 @@ int Monitor(mddev_dev_t devlist,
 					st->spare_group = NULL;
 					statelist = st;
 					alert("NewArray", st->devname, NULL, mailaddr, alert_cmd);
+					new_found = 1;
 				}
 		}
 		/* If an array has active < raid && spare == 0 && spare_group != NULL
@@ -372,8 +389,12 @@ int Monitor(mddev_dev_t devlist,
 						close(fd2);
 					}
 			}
-					    
-		sleep(period);
+		if (!new_found) {
+			if (oneshot)
+				break;
+			else
+				sleep(period);
+		}
 	}
 	return 0;
 }
@@ -399,7 +420,9 @@ static void alert(char *event, char *dev, char *disc, char *mailaddr, char *cmd)
 			exit(2);
 		}
 	}
-	if (mailaddr && strncmp(event, "Fail", 4)==0) {
+	if (mailaddr && 
+	    (strncmp(event, "Fail", 4)==0 || 
+	     strncmp(event, "Degrade", 7)==0)) {
 		FILE *mp = popen(Sendmail, "w");
 		if (mp) {
 			char hname[256];
@@ -415,7 +438,7 @@ static void alert(char *event, char *dev, char *disc, char *mailaddr, char *cmd)
 			fprintf(mp, "A %s event had been detected on md device %s.\n\n", event, dev);
 
 			if (disc)
-				fprintf(mp, "It could be related to componenet device %s.\n\n", disc);
+				fprintf(mp, "It could be related to component device %s.\n\n", disc);
 
 			fprintf(mp, "Faithfully yours, etc.\n");
 			fclose(mp);
